local ADDON, ns = ...
local FakeKeystones = LibStub("AceAddon-3.0"):NewAddon(ADDON, "AceConsole-3.0","AceEvent-3.0","AceHook-3.0","AceComm-3.0")
local L = ns.L

local mapIDs = { -- C_ChallengeMode.GetMapTable()
	375,
	376,
	377,
	378,
	379,
	380,
	381,
	382,
}

local oldMapIDs = {2,56,57,58,59,60,76,77,78,161,163,164,165,166,167,168,169,197,198,199,200,206,207,208,209,210,227,233,234,239,244,245,246,247,248,249,250,251,252,353,369,370}

local keystoneItemIDs = {
	[138019] = EXPANSION_NAME6,
	[158923] = EXPANSION_NAME7,
	[180653] = EXPANSION_NAME8,
	[187786] = L["Timeworn Keystone"],
}

local mapList, oldMapList, affixList, entries = {}, {}, {}

local defaults = {
	["mapId"] = 376,
	["mythicLevel"] = 20,
	["affixId1"] = 9,
	["affixId2"] = 11,
	["affixId3"] = 4,
	["affixId4"] = 120,
	["angryKeystone"] = true,
	["challengesFrame"] = false,
	["weeklyLevel"] = 20,
	["dungeons"] = {},
	["currentExpansion"] = true,
	["keystoneItemID"] = 180653,
}

function FakeKeystones:LoadCurrentAffixes()
	local affixes = C_MythicPlus.GetCurrentAffixes()
	if not affixes then return end
	for index in ipairs(affixes) do
		self.db["affixId"..index] = affixes[index].id
	end
end

function FakeKeystones:PrintKeystone()
	self:SendKeystone()

	local mapId, level, affixId1, affixId2, affixId3, affixId4, keystoneItemID = self.db["mapId"], self.db["mythicLevel"], self.db["affixId1"], self.db["affixId2"], self.db["affixId3"], self.db["affixId4"], self.db["keystoneItemID"]
	local mapName = C_ChallengeMode.GetMapUIInfo(mapId)
	local keystone = mapName and string.format(CHALLENGE_MODE_KEYSTONE_HYPERLINK, mapName, level)
	if not keystone then return end

	_G.DEFAULT_CHAT_FRAME:AddMessage(L["Keystone Link: "]..string.format("|cffa335ee|Hkeystone:%d:%d:%d:%d:%d:%d:%d|h[%s]|h|r", keystoneItemID, mapId, level, affixId1, affixId2, affixId3, affixId4, keystone))
end

local function get_function(info)
	return FakeKeystones.db[info[#info]]
end

local function set_function(info, val)
	FakeKeystones.db[info[#info]] = val
end

local function getMapList()
	return FakeKeystones.db["currentExpansion"] and mapList or oldMapList
end

local options = {
	type = "group",
	name = "Fake Keystones",
	args = {
		space = {
			order = 0,
			type = "description",
			name = " ",
			width = "full",
		},
	},
}

options.args["keystone"] = {
	order = 1,
	name = L["Mythic Keystone"],
	type = "group",
	args = {
		enable = {
			name = L["Fake AngryKeystones"],
			desc = L["It will replace AngryKeystones and send keystone info to your party automatically."],
			order = 1,
			type = "toggle",
			get = function() return FakeKeystones.db["angryKeystone"] end,
			set = function(_, val)
				FakeKeystones.db["angryKeystone"] = val 
			end,
			width = "double",
		},
		description = {
			name = "",
			order = 2,
			type = "description",
		},
		keystoneItemID = {
			name = L["Keystone Type"],
			order = 3,
			type = "select",
			values = keystoneItemIDs,
			get = get_function,
			set = set_function,
			width = "double",
		},
		mapId = {
			name = DUNGEONS,
			order = 4,
			type = "select",
			values = getMapList,
			get = get_function,
			set = function(_, val)
				if val == 888 then
					FakeKeystones.db["currentExpansion"] = true
					FakeKeystones.db["mapId"] = mapIDs[1]
				elseif val == 999 then
					FakeKeystones.db["currentExpansion"] = false
					FakeKeystones.db["mapId"] = oldMapIDs[1]
				else
					FakeKeystones.db["mapId"] = val
				end
			end,
			width = "double",
		},
		mythicLevel = {
			name = L["Level"],
			order = 5,
			type = "range",
			min = 2,
			max = 35,
			step = 1,
			get = get_function,
			set = set_function,
			width = "double"
		},
		affixId1 = {
			name = L["Affix"].." 1",
			order = 6,
			type = "select",
			values = affixList,
			get = get_function,
			set = set_function,
		},
		affixId2 = {
			name = L["Affix"].." 2",
			order = 7,
			type = "select",
			values = affixList,
			get = get_function,
			set = set_function,
		},
		affixId3 = {
			name = L["Affix"].." 3",
			order = 8,
			type = "select",
			values = affixList,
			get = get_function,
			set = set_function,
		},
		affixId4 = {
			name = L["Affix"].." 4",
			order = 9,
			type = "select",
			values = affixList,
			get = get_function,
			set = set_function,
		},
		printKeystone = {
			name = L["Print"],
			desc = L["Print keystone link and send keystone info."],
			order = 10,
			type = "execute",
			func = function() FakeKeystones:PrintKeystone() end,
		},
		reset = {
			name = RESET,
			desc = L["Reset to current affixes"],
			order = 11,
			type = "execute",
			func = function() FakeKeystones:LoadCurrentAffixes() end,
		},
	}
}

local function getKeystone(current)
	local keystoneMapID = FakeKeystones.db["mapId"]
	local keystoneLevel = FakeKeystones.db["mythicLevel"]
	if current and type(current) == "boolean" then
		keystoneMapID = C_MythicPlus.GetOwnedKeystoneChallengeMapID()
		keystoneLevel = C_MythicPlus.GetOwnedKeystoneLevel()
	end
	if current and type(current) == "number" then
		keystoneMapID = C_MythicPlus.GetOwnedKeystoneChallengeMapID()
		keystoneLevel = C_MythicPlus.GetOwnedKeystoneLevel()
		if keystoneLevel then keystoneLevel = keystoneLevel + current end
	end

	local message = "0"
	if keystoneLevel and keystoneMapID then
		message = string.format("%d:%d", keystoneMapID, keystoneLevel)
	end

	return message
end

function FakeKeystones:SendKeystone()
	if not FakeKeystones.db["angryKeystone"] then return end
	local message = getKeystone()
	FakeKeystones:SendCommMessage("AngryKeystones", "Schedule|"..message, "PARTY")
	FakeKeystones:SendCommMessage("FakeKeystones", getKeystone(true), "PARTY")
end

function FakeKeystones:ShowConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize(ADDON, 600, 420)
	LibStub("AceConfigDialog-3.0"):Open(ADDON)
end

function FakeKeystones:OnInitialize()
	self.db = FakeKeystones_DB or {}
	FakeKeystones_DB = self.db

	for key in pairs(self.db) do
		if defaults[key] == nil then
			self.db[key] = nil
		end
	end

	for key, value in pairs(defaults) do
		if self.db[key] == nil then
			if type(value) == "table" then
				self.db[key] = {}
				for k in pairs(value) do
					self.db[key][k] = value[k]
				end
			else
				self.db[key] = value
			end
		end
	end

	for _, id in ipairs(mapIDs) do
		mapList[id] = C_ChallengeMode.GetMapUIInfo(id)
	end
	mapList[999] = "< "..L["Old Expansion"]

	for _, id in ipairs(oldMapIDs) do
		oldMapList[id] = C_ChallengeMode.GetMapUIInfo(id)
	end
	oldMapList[888] = "> "..EXPANSION_NAME8

	for i = 1, 255 do
		local name = C_ChallengeMode.GetAffixInfo(i)
		if name then
			affixList[i] = name
		end
	end
	affixList[0] = NONE

	self:ChallengesOptions_Init()

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(ADDON, options)
	self:RegisterChatCommand("fks", "ShowConfig")
	self:RegisterChatCommand("fakekeystones", "ShowConfig")
	self:RegisterComm("FakeKeystones")
	self:RegisterEvent("ADDON_LOADED")
end

function FakeKeystones:OnEnable()
	self:LoadCurrentAffixes()

	if IsAddOnLoaded("AngryKeystones") then 
		local mod = _G.AngryKeystones.Modules.Schedule
		if mod then
			self:RawHook(mod, "SendCurrentKeystone", function()
				if self.db["angryKeystone"] then
					self:SendKeystone()
				else
					self.hooks[mod].SendCurrentKeystone(mod)
				end
			end, true)
		end
	else
		self:RegisterComm("AngryKeystones")
		self:RegisterEvent("GROUP_ROSTER_UPDATE", "SendKeystone")
		self:RegisterEvent("CHALLENGE_MODE_START", "SendKeystone")
		self:RegisterEvent("CHALLENGE_MODE_COMPLETED", "SendKeystone")
	end
end

function FakeKeystones:OnCommReceived(prefix, message, distribution, sender)
	if string.match(message, "quest") then
		self.SendKeystone()
	end
end

function FakeKeystones:ADDON_LOADED(event, addon)
	if addon == "Blizzard_ChallengesUI" then
		self:SecureHook("ChallengesFrame_Update", "ChallengesFrame_Hook")
		self:UnregisterEvent("ADDON_LOADED")
	end	
end

local function map_get_function(info)
	return FakeKeystones.db["dungeons"][info[#info]]
end

local function map_set_function(info, val)
	FakeKeystones.db["dungeons"][info[#info]] = val
end

local challenges_options = {
	order = 2,
	name = L["Challenges Frame"],
	type = "group",
	args = {
		enable = {
			name = L["Modify Challenges Frame"],
			order = 1,
			type = "toggle",
			get = function() return FakeKeystones.db["challengesFrame"] end,
			set = function(_, val)
				FakeKeystones.db["challengesFrame"] = val
				if ChallengesFrame_Update then
					ChallengesFrame_Update(ChallengesFrame)
				end
			end,
			width = "double",
		},
		description1 = {
			name = "",
			order = 2,
			type = "description",
		},
		weeklyLevel = {
			name = L["Best M+ Level"],
			order = 3,
			type = "range",
			min = 2,
			max = 35,
			step = 1,
			get = get_function,
			set = set_function,
			width = "double",
		},
		description2 = {
			name = "\n"..L["Dungeons :"],
			order = 4,
			type = "description",
		},
	}
}

local map_options_table = {
	type = "range",
	min = 2,
	max = 35,
	step = 1,
	get = map_get_function,
	set = map_set_function,
}

local score_options_table = {
	type = "range",
	name = L["Score"],
	min = 0,
	max = 500,
	step = 1,
	get = map_get_function,
	set = map_set_function,
}

function FakeKeystones:ChallengesOptions_Init()
	local order = 5

	for i=1, #mapIDs do
		local id = mapIDs[i]
		local map = tostring(id)
		self.db["dungeons"][map] = self.db["dungeons"][map] or 20

		challenges_options.args[map] = {}
		challenges_options.args[map].name = mapList[id]
		challenges_options.args[map].order = order

		for k, v in pairs(map_options_table) do
			challenges_options.args[map][k] = v
		end

		order = order + 1

		local score = map.."score"
		self.db["dungeons"][score] = self.db["dungeons"][score] or 300

		challenges_options.args[score] = {}
		challenges_options.args[score].order = order

		for k, v in pairs(score_options_table) do
			challenges_options.args[score][k] = v
		end

		order = order + 1
	end

	options.args["ChallengesFrame"] = challenges_options
end

local LEGENDARY_COMPLETION_LEVEL = 15
local EPIC_COMPLETION_LEVEL = 10
local RARE_COMPLETION_LEVEL = 7
local UNCOMMON_COMPLETION_LEVEL = 4
local COMMON_COMPLETION_LEVEL = 2

local function getRunQualityBasedOnLevel(level)
	if (level >= LEGENDARY_COMPLETION_LEVEL) then
		return Enum.ItemQuality.Legendary 
	elseif (level < LEGENDARY_COMPLETION_LEVEL and level >= EPIC_COMPLETION_LEVEL) then
		return Enum.ItemQuality.Epic
	elseif (level < EPIC_COMPLETION_LEVEL and level >= RARE_COMPLETION_LEVEL) then
		return Enum.ItemQuality.Rare
	elseif (level < RARE_COMPLETION_LEVEL and level >= UNCOMMON_COMPLETION_LEVEL) then
		return Enum.ItemQuality.Uncommon
	elseif (level < UNCOMMON_COMPLETION_LEVEL and level >= COMMON_COMPLETION_LEVEL) then
		return Enum.ItemQuality.Common
	else 
		return Enum.ItemQuality.Poor
	end
end 

local function updateCurrentText(self, text)
	if self.setting or not FakeKeystones.db["challengesFrame"] then return end

	local mod = AngryKeystones.Modules.Schedule
	local locale = AngryKeystones.Modules.Locale
	local keystoneString = locale and locale:Get("currentKeystoneText")
	local mapName = C_ChallengeMode.GetMapUIInfo(FakeKeystones.db["mapId"])
	local keystoneName = mapName and string.format("%s (%d)", mapName, FakeKeystones.db["mythicLevel"])

	if keystoneName and keystoneString then
		self.setting = true
		self:SetText(string.format(keystoneString, keystoneName))
		self.setting = nil
	end
end

local function showObject(self)
	if not FakeKeystones.db["challengesFrame"] then return end
	self:Show()
end

local function updateGuildName(self, text)
	if self.setting or not FakeKeystones.db["challengesFrame"] then return end

	local name = UnitName("player")
	local _, classFilename = UnitClass("player")
	local classColorStr = RAID_CLASS_COLORS[classFilename].colorStr

	self.setting = true
	self:SetText(format(CHALLENGE_MODE_GUILD_BEST_LINE_YOU, classColorStr, name))
	self.setting = nil
end

local function updateGuildLevel(self, text)
	if self.setting or not FakeKeystones.db["challengesFrame"] then return end

	local level = FakeKeystones.db["weeklyLevel"]

	self.setting = true
	self:SetText(level)
	self.setting = nil
end

function FakeKeystones:ChallengesFrame_Hook()
	if not FakeKeystones.db["challengesFrame"] then
		if entries then
			for i = 1, #entries do
				entries[i]:Show()
			end
		end
		return
	end

	local sortedMaps = {}
	for i = 1, #mapIDs do
		local id = mapIDs[i]
		local map = tostring(id)
		local level = self.db["dungeons"][map]
		local score = self.db["dungeons"][map.."score"]

		tinsert(sortedMaps, { id = id, level = level, dungeonScore = score})
	end

	table.sort(sortedMaps, 
	function(a, b)
		return a.dungeonScore > b.dungeonScore
	end)
	
	local totalScore = 0

	for i = 1, #sortedMaps do
		local frame = ChallengesFrame.DungeonIcons[i]
		frame:SetUp(sortedMaps[i], i == 1)
		
		local score = sortedMaps[i].dungeonScore
		totalScore = totalScore + score

		local color = C_ChallengeMode.GetSpecificDungeonOverallScoreRarityColor(score)
		frame.HighestLevel:SetTextColor(color.r, color.g, color.b)
	end

	local _, _, _, _, backgroundTexture = C_ChallengeMode.GetMapUIInfo(sortedMaps[1].id)
	if (backgroundTexture ~= 0) then
		ChallengesFrame.Background:SetTexture(backgroundTexture)
	end

	ChallengesFrame.WeeklyInfo:SetUp(true, sortedMaps[1])

	local child = ChallengesFrame.WeeklyInfo.Child
	local activeChest = child.WeeklyChest
	activeChest.Icon:SetAtlas("mythicplus-greatvault-complete", TextureKitConstants.UseAtlasSize)
	activeChest.Highlight:SetAtlas("mythicplus-greatvault-complete", TextureKitConstants.UseAtlasSize)
	activeChest.RunStatus:SetText(MYTHIC_PLUS_COMPLETE_MYTHIC_DUNGEONS)
	activeChest.AnimTexture:Hide()
	activeChest:Show()

	local color = C_ChallengeMode.GetDungeonScoreRarityColor(totalScore)
	if(color) then 
		ChallengesFrame.WeeklyInfo.Child.DungeonScoreInfo.Score:SetVertexColor(color.r, color.g, color.b)
	end 
	ChallengesFrame.WeeklyInfo.Child.DungeonScoreInfo.Score:SetText(totalScore)
	ChallengesFrame.WeeklyInfo.Child.DungeonScoreInfo:SetShown(true)

	child.ThisWeekLabel:Show()
	child.Description:Hide()

	if IsAddOnLoaded("AngryKeystones") then
		local mod = AngryKeystones.Modules.Schedule
		mod.KeystoneText:Show()
		updateCurrentText(mod.KeystoneText)

		if not self.AngryKeystonesHooked then
			hooksecurefunc(mod.KeystoneText, "Hide", showObject)
			hooksecurefunc(mod.KeystoneText, "SetText", updateCurrentText)
			self.AngryKeystonesHooked = true
		end
	end

	if entries then
		for i = 1, #entries do
			if i ~= 1 then
				entries[i]:Hide()
			end
		end
	else
		for i = 1, ChallengesFrame:GetNumChildren() do
			local child = select(i, ChallengesFrame:GetChildren())
			if child and child.entries and child.entries[1] and child.entries[1].CharacterName then
				entries = child.entries
				for i = 1, #entries do
					local entry = entries[i]
					if i == 1 then
						hooksecurefunc(entry.CharacterName, "SetText", updateGuildName)
						hooksecurefunc(entry.Level, "SetText", updateGuildLevel)
						entry:Show()
						entry.CharacterName:SetText("")
						entry.Level:SetText("")
					else
						entry:Hide()
					end
				end
				break
			end
		end
	end
end